==> jdreact-core-web-0.0.1.tgz: package/Libraries/ScrollView/ScrollViewStickyHeader.js <==
import PropTypes from 'prop-types';
import React, { PureComponent } from 'react';
import ReactDOM from 'react-dom';
import View from '../View/View';
import StyleSheet from '../StyleSheet/StyleSheet';

class ScrollViewStickyHeader extends PureComponent {
  constructor(props) {
    super(props);
    this.state = {
      headerFixedRect: null,
    };
    this.layout = null;
    this.clientRect = null;
    this.parentScrollTop = 0;
    this.offsetTop = undefined;
  }

  static propTypes = {
    stickyHeaderIndexChange: PropTypes.func
  };
  componentDidMount() {
    document.addEventListener('containerLayout', this._setClientRect, false);
  }

  componentWillUnmount() {
    document.addEventListener('containerLayout', this._setClientRect, false);
  }

  render() {

    return (
      <View onLayout={this._onLayout} className="sticky-header">
        <div ref="stickHeader">{this.props.children}</div>
        <div ref="stickHeaderHolder" />
      </View>
    )
  }

  componentDidUpdate() {
    const headerFixedRect = this.state.headerFixedRect
    let stickHeader = (ReactDOM.findDOMNode(this.refs.stickHeader));
    let stickHeaderHolder = (ReactDOM.findDOMNode(this.refs.stickHeaderHolder));
    if (!!headerFixedRect) {
      let offsetTop = headerFixedRect.top;
      if (this.offsetTop !== undefined) {
        offsetTop = this.offsetTop;
      }
      stickHeader.style.position = "fixed";
      stickHeader.style.zIndex = 1;
      stickHeader.style.top = `${offsetTop}px`;
      stickHeader.style.left = `${headerFixedRect.left}px`;
      stickHeader.style.height = `${headerFixedRect.height}px`;
      stickHeader.style.width = `${headerFixedRect.width}px`;
      stickHeaderHolder.style.height = `${headerFixedRect.height}px`;
      stickHeaderHolder.style.width = `${headerFixedRect.width}px`;
    } else {
      if (!!stickHeader) {
        stickHeader.style.position = "";
        stickHeaderHolder.style.height = `0px`;
        stickHeaderHolder.style.width = `0px`;
      }
    }
  }
  _setClientRect = () => {
    const layout = this.layout;
    const rect = ReactDOM.findDOMNode(this).getBoundingClientRect();
    const clientRect = {
      top: rect.top - layout.y + this.parentScrollTop,
      bottom: rect.bottom - layout.y + this.parentScrollTop,
      width: rect.width,
      height: rect.height,
      left: rect.left,
      right: rect.right,
    };
    this.clientRect = clientRect;
    const headerFixedRect = this.state.headerFixedRect;
    if (headerFixedRect) {
      const keys = Object.keys(headerFixedRect);
      for (let i = 0; i < keys.length; i++) {
        const key = keys[i];
        if (headerFixedRect[key] !== clientRect[key]) {
          this.setState({
            headerFixedRect: clientRect,
          });
          break;
        }
      }
    }
  }

  _onLayout = (evt) => {
    this.layout = evt.nativeEvent.layout;
    this._setClientRect();
    this.props.onLayout && this.props.onLayout(this.layout);
  }

  checkHeaderPosition = (parentScrollTop, offsetTop) => {
    this.parentScrollTop = parentScrollTop;
    if(offsetTop !== undefined) {
      this.offsetTop = offsetTop;
    }
    const { layout, nextLayout } = this;
    if (parentScrollTop >= (layout.y - layout.height - layout.height) && (typeof nextLayout === 'undefined' ? true : parentScrollTop < nextLayout.y)) {
      !this.state.headerFixedRect && this.setState({
        headerFixedRect: this.clientRect
      });
      this.props.stickyHeaderIndexChange && this.props.stickyHeaderIndexChange(layout.y);
    } else {
      this.state.headerFixedRect && this.setState({
        headerFixedRect: null,
      });
    }
  }

  setNextScrollY = (nextLayout) => {
    this.nextLayout = nextLayout;
  }
}

const styles = StyleSheet.create({
  stickyHeader: {
    zIndex: 1,
    backgroundColor: '#FF0',
    position: 'fixed',
  },
});

export default ScrollViewStickyHeader;

==> jdreact-core-web-0.0.3.tgz: package/Libraries/ScrollView/ScrollViewStickyHeader.js <==
import PropTypes from 'prop-types';
import React, { PureComponent } from 'react';
import ReactDOM from 'react-dom';
import View from '../View/View';
import StyleSheet from '../StyleSheet/StyleSheet';

class ScrollViewStickyHeader extends PureComponent {
  constructor(props) {
    super(props);
    this.state = {
      headerFixedRect: null,
    };
    this.layout = null;
    this.clientRect = null;
    this.parentScrollTop = 0;
    this.offsetTop = undefined;
  }

  static propTypes = {
    stickyHeaderIndexChange: PropTypes.func
  };
  componentDidMount() {
    document.addEventListener('containerLayout', this._setClientRect, false);
  }

  componentWillUnmount() {
    document.addEventListener('containerLayout', this._setClientRect, false);
  }

  render() {

    return (
      <View onLayout={this._onLayout} className="sticky-header">
        <div ref="stickHeader">{this.props.children}</div>
        <div ref="stickHeaderHolder" />
      </View>
    )
  }

  componentDidUpdate() {
    const headerFixedRect = this.state.headerFixedRect
    let stickHeader = (ReactDOM.findDOMNode(this.refs.stickHeader));
    let stickHeaderHolder = (ReactDOM.findDOMNode(this.refs.stickHeaderHolder));
    if (!!headerFixedRect) {
      let offsetTop = headerFixedRect.top;
      if (this.offsetTop !== undefined) {
        offsetTop = this.offsetTop;
      }
      stickHeader.style.position = "fixed";
      stickHeader.style.zIndex = 1;
      stickHeader.style.top = `${offsetTop}px`;
      stickHeader.style.left = `${headerFixedRect.left}px`;
      stickHeader.style.height = `${headerFixedRect.height}px`;
      stickHeader.style.width = `${headerFixedRect.width}px`;
      stickHeaderHolder.style.height = `${headerFixedRect.height}px`;
      stickHeaderHolder.style.width = `${headerFixedRect.width}px`;
    } else {
      if (!!stickHeader) {
        stickHeader.style.position = "";
        stickHeaderHolder.style.height = `0px`;
        stickHeaderHolder.style.width = `0px`;
      }
    }
  }
  _setClientRect = () => {
    const layout = this.layout;
    const rect = ReactDOM.findDOMNode(this).getBoundingClientRect();
    const clientRect = {
      top: rect.top - layout.y + this.parentScrollTop,
      bottom: rect.bottom - layout.y + this.parentScrollTop,
      width: rect.width,
      height: rect.height,
      left: rect.left,
      right: rect.right,
    };
    this.clientRect = clientRect;
    const headerFixedRect = this.state.headerFixedRect;
    if (headerFixedRect) {
      const keys = Object.keys(headerFixedRect);
      for (let i = 0; i < keys.length; i++) {
        const key = keys[i];
        if (headerFixedRect[key] !== clientRect[key]) {
          this.setState({
            headerFixedRect: clientRect,
          });
          break;
        }
      }
    }
  }

  _onLayout = (evt) => {
    this.layout = evt.nativeEvent.layout;
    this._setClientRect();
    this.props.onLayout && this.props.onLayout(this.layout);
  }

  checkHeaderPosition = (parentScrollTop, offsetTop) => {
    this.parentScrollTop = parentScrollTop;
    if(offsetTop !== undefined) {
      this.offsetTop = offsetTop;
    }
    const { layout, nextLayout } = this;
    if (parentScrollTop > layout.y && (typeof nextLayout === 'undefined' ? true : parentScrollTop < (nextLayout.y + nextLayout.height))) {
      !this.state.headerFixedRect && this.setState({
        headerFixedRect: this.clientRect
      });
      this.props.stickyHeaderIndexChange && this.props.stickyHeaderIndexChange(layout.y);
    } else {
      this.state.headerFixedRect && this.setState({
        headerFixedRect: null,
      });
    }
  }

  setNextScrollY = (nextLayout) => {
    this.nextLayout = nextLayout;
  }
}

const styles = StyleSheet.create({
  stickyHeader: {
    zIndex: 1,
    backgroundColor: '#FF0',
    position: 'fixed',
  },
});

export default ScrollViewStickyHeader;
